<%@ page contentType="text/javascript" %>
/**
 * Validates whether the checkbox has options selected within a range
 * Or if a checkbox has at least a certain amount of options selected
 * Or has exactly an amount of options selected
 * Receives the rule definition and its scope.
 * scope.$parent will contain the form to enable watch and access to other fields in the same step
 * ctrls will be an array of controllers, first the ffFormController and then the ngModel of the associated input
 *
 * @param {object} rule current rule definition
 * @param {object} scope current input scope
 * @param {object} el current html element
 * @param {array} attr attributes of the html element
 * @param {array} ctrls ctrls[0] = FFFormController ctrls[1]=ngModel
 */

ff.validationRules.numberOfCheckboxes = function(rule, scope, el, attr, ctrls) {
    'use strict';

    var checkboxesMin = parseInt(rule.checkboxesmin);
    var checkboxesMax = parseInt(rule.checkboxesmax);
    var input = scope.input;
    return function(modelValue, viewValue) {
        //This is a checkbox array we check the whole input.value
        if(!_.isUndefined(input) && !_.isEmpty(input)) {
            var checkedBoxes = input.value.length;
            var validity = true;
            if (!_.isUndefined(checkboxesMin) && !_.isUndefined(checkboxesMax)) {
                if (checkboxesMin === checkboxesMax && checkedBoxes !== checkboxesMin) {
                    validity = false;
                }
                else if (checkedBoxes < checkboxesMin || checkedBoxes > checkboxesMax) {
                    // Not valid
                    validity = false;
                }
            } else if (!_.isUndefined(checkboxesMin)) {
                if (checkboxesMin < checkedBoxes) {
                    validity = false;
                }
            } else if (!_.isUndefined(checkboxesMax)) {
                if (checkboxesMax > checkedBoxes) {
                    validity = false;
                }
            }
            return validity;
        }
        return false;
    };
};