<%@ page contentType="text/javascript" %>
/**
 * Validates that the length of the value is between the specified range or meets the required limit.
 * Receives the rule definition and its scope.
 * scope.$parent will contain the form to enable watch and access to other fields in the same step
 * ctrls will be an array of controllers, first the ffFormController and then the ngModel of the associated input
 *
 * @param {object} rule current rule definition
 * @param {object} scope current input scope
 * @param {object} el current html element
 * @param {array} attr attributes of the html element
 * @param {array} ctrls ctrls[0] = FFFormController ctrls[1]=ngModel
 */

ff.validationRules.rangeLength = function (rule, scope, el, attr, ctrls) {

    'use strict';

    var model = ctrls[1];
    var min = parseInt(rule.rangelengthmin);
    var max = parseInt(rule.rangelengthmax);

    return function(modelValue, viewValue){
        if (viewValue === null) {
            return true;
        }
        if(_.isUndefined(viewValue) || !_.isString(viewValue) || _.isEmpty(viewValue)) {
            return false;
        }
        var length = parseInt(viewValue.length);

        if(length == 0){
            return true;
        }
        if(!model.$isEmpty(min) && !model.$isEmpty(max)){
            //min and max value is specified
            if(min == max){
                //value has to be the exact length.
                return length == min
            }else {
                //validate that value is between range
                return (length >= min && length <= max);
            }
        }else if(model.$isEmpty(min) && !model.$isEmpty(max)){
            //min is not specified and max is specified, validate if view value is ln or eq to max
            return length <= max;
        }else if(!model.$isEmpty(min) && model.$isEmpty(max)){
            //max is not specified and min is specified, validate if view value is gt or eq to min
            return length >= min;
        }

    };
};