/**
 * For each case of validation this function returns an object:
 * validationType:{
 * 		errorInfo: "" 					//Index or step name depending on validation type or a set of values separated by "|" as required by i18nService.format() from MF
 * 		validationMessage: ""
 *		toastId: ""
 * }
 *
 * The toast id will be associated with a toast and upon display will replace toasts (if any) for a given validation
 * type.
 *
 */
onmessage = function (e) {
	var hasInvalidSteps = false;
	var messages = {};

	if (e.data.form.steps.length < 1) {
		hasInvalidSteps = true;
		messages["noStep"] = {};
		messages["noStep"].errorInfo = "";
		messages["noStep"].validationMessage = "angular.ffFormBuilderDirective.message.noStep";
		messages["noStep"].toastId = "ns";
	}

	//Check for invalid steps
	for (var i in e.data.form.steps) {
		//Send empty step message and step name
		if (e.data.form.steps[i].inputs.length < 1) {
			hasInvalidSteps = true;
			messages["emptyStep"] = {};
			messages["emptyStep"].errorInfo = e.data.form.steps[i].label !== "" ? "'" + e.data.form.steps[i].label + "'" : "'name is empty'";
			messages["emptyStep"].stepIndex = i;
			messages["emptyStep"].validationMessage = "angular.ffFormBuilderDirective.message.emptyStep";
			messages["emptyStep"].toastId = "es";
		}
		//Send invalid step name message and step index
		if (e.data.form.steps[i].label === '') {
			hasInvalidSteps = true;
			messages["emptyStepName"] = {};
			messages["emptyStepName"].errorInfo = i;
			messages["emptyStepName"].validationMessage = "angular.ffFormBuilderDirective.message.invalidStepName";
			messages["emptyStepName"].toastId = "esn";
		}
	}
	//Check if actions step is invalid
	if (e.data.form.actions.length < 1) {
		hasInvalidSteps = true;
		messages["noAction"] = {};
		messages["noAction"].errorInfo = "action";
		messages["noAction"].validationMessage = "angular.ffFormBuilderDirective.message.missingAction";
		messages["noAction"].toastId = "na";
	}
	//Check if actions on form have required fields
	var invalidAction = false;
	//Email regexp
	var emailRegexp = /^[A-Z0-9._%+-]{1,64}@(?:[A-Z0-9-]{1,63}\.){1,125}[A-Z]{2,63}$/i;
	for (var i in e.data.form.actions) {
		if ('requiredFields' in e.data.form.actions[i] && e.data.form.actions[i].requiredFields.length > 0) {
			//Validate the required fields.
			for (var j in e.data.form.actions[i].requiredFields) {
				//get the current value of the required field from the action
				var value = e.data.form.actions[i][e.data.form.actions[i].requiredFields[j].key];
				var isValid = true;
				switch(e.data.form.actions[i].requiredFields[j].value) {
					case 'number' :
						if (value === undefined || value === '' || value === null || isNaN(value)) {
							isValid = false;
						}
						break;
					case 'positiveNumber' :
						if (value === undefined || value === '' || value === null || isNaN(value) || value < 0) {
							isValid = false;
						}
						break;
					case 'email' :
						if (!emailRegexp.test(value)) {
							isValid = false;
						}
						break;
					case 'any' :
					default :
						if (value === undefined || value === "" || value === null) {
							isValid = false;
						}
						break;
				}
				if (!isValid) {
					invalidAction = true;
					break;
				}
			}
			if(invalidAction) {
				hasInvalidSteps = true;
				messages['invalidActionFields'] = {};
				messages['invalidActionFields'].errorInfo = e.data.form.actions[i].label;
				messages['invalidActionFields'].validationMessage = 'angular.ffFormBuilderDirective.message.invalidActionFields';
				messages['invalidActionFields'].toastId = 'iaf';
				messages['invalidActionFields'].actionIndex = i;
				invalidAction = false;
			}
		}
	}

	hasInvalidSteps = (hasInvalidSteps) ? true : validateLogics(e.data.form, messages);

	//Pass flag value to disable/enable UI elements and callback(s) with messages
	postMessage({
		flag: "containsInvalidStep",
		flagValue: hasInvalidSteps,
		callbacks: [
			{
				callback: 'notifyUserOfInvalidStep',
				args: messages
			}
		]
	});
	//Remove thread!!!
	close();
};

var validateLogics = function (form, messages) {
	var hasInvalidSteps = false;
	//1st step contains > n-1 logics applications
	var numberOfLogicsOnFirstStep = 0;
	for (var i in form.steps[0].inputs) {
		if (form.steps[0].inputs[i].logics !== undefined) {
			numberOfLogicsOnFirstStep++;
		}
	}

	if (numberOfLogicsOnFirstStep === form.steps[0].inputs.length) {
		hasInvalidSteps = true;
		messages["logicsFirstStepError"] = {};
		messages["logicsFirstStepError"].errorInfo = form.steps[0].label;
		messages["logicsFirstStepError"].validationMessage = "angular.ffFormBuilderDirective.message.firstStepLogicsError";
		messages["logicsFirstStepError"].toastId = "logicsFirstStepError";
		return hasInvalidSteps;
	}

	//Build input map
	var inputMap = {};
	for (var i in form.steps) {
		for (var j in form.steps[i].inputs) {
			inputMap[form.steps[i].inputs[j].name] = {stepNumber: i, stepName: form.steps[i].label};
		}
	}

	//Validate logics at a given step against inputMap
	//Input number in inputMap must be <= step number of the step where that input is used as source of logics.
	for (var i in form.steps) {
		for (var j in form.steps[i].inputs) {
			if (form.steps[i].inputs[j].logics !== undefined) {
				for (var k in form.steps[i].inputs[j].logics) {
					if (!(form.steps[i].inputs[j].logics[k].fieldName in inputMap)) {
						hasInvalidSteps = true;
						messages["logicsSourceMissing"] = {};
						messages["logicsSourceMissing"].errorInfo = form.steps[i].inputs[j].name
						+ "|" + form.steps[i].label
						+ "|" + form.steps[i].inputs[j].logics[k].fieldName;
						messages["logicsSourceMissing"].validationMessage = "angular.ffFormBuilderDirective.message.logicSourceMissing";
						messages["logicsSourceMissing"].toastId = "logicsSourceMissing";
						return hasInvalidSteps;
					}
					else if (parseInt(inputMap[form.steps[i].inputs[j].logics[k].fieldName].stepNumber) > i) {
						hasInvalidSteps = true;
						messages["logicsUnreachableSource"] = {};
						messages["logicsUnreachableSource"].errorInfo = form.steps[i].inputs[j].name
						+ "|" + form.steps[i].label
						+ "|" + form.steps[i].inputs[j].logics[k].fieldName
						+ "|" + inputMap[form.steps[i].inputs[j].logics[k].fieldName].stepName;
						messages["logicsUnreachableSource"].validationMessage = "angular.ffFormBuilderDirective.message.logicSourceNotReachable";
						messages["logicsUnreachableSource"].toastId = "logicsUnreachableSource";
						return hasInvalidSteps;
					}
				}
			}
		}
	}

	//Test for cycles
	hasInvalidSteps = (hasInvalidSteps) ? true : logicsCycleTest(form, inputMap, messages);
	hasInvalidSteps = (hasInvalidSteps) ? true : emptyLogicsTest(form, messages);

	return hasInvalidSteps;
};

var logicsCycleTest = function (form, inputMap, messages) {
	var hasInvalidSteps = false;
	var sourceTargetMap = [];
	for (var i in form.steps) {
		var step = {
			targetSourcePairs: [],
			stepName: form.steps[i].label
		};
		for (var j in form.steps[i].inputs) {
			if (form.steps[i].inputs[j].logics !== undefined) {
				for (var k in form.steps[i].inputs[j].logics) {
					if (parseInt(inputMap[form.steps[i].inputs[j].logics[k].fieldName].stepNumber) == i) {
						step.targetSourcePairs.push({
							target: form.steps[i].inputs[j].name,
							source: form.steps[i].inputs[j].logics[k].fieldName
						});
					}
				}
			}
		}
		sourceTargetMap.push(step);
	}

	for (var step in sourceTargetMap) {
		for (var l1 in sourceTargetMap[step].targetSourcePairs) {
			for (var l2 in sourceTargetMap[step].targetSourcePairs) {
				if (l1 != l2) {
					if (sourceTargetMap[step].targetSourcePairs[l2].target === sourceTargetMap[step].targetSourcePairs[l1].source
						&& sourceTargetMap[step].targetSourcePairs[l2].source === sourceTargetMap[step].targetSourcePairs[l1].target) {
						hasInvalidSteps = true;
						messages["logicsCycleDetected"] = {};
						messages["logicsCycleDetected"].errorInfo = sourceTargetMap[step].stepName
						+ "|" + sourceTargetMap[step].targetSourcePairs[l1].target
						+ "|" + sourceTargetMap[step].targetSourcePairs[l1].source;
						messages["logicsCycleDetected"].validationMessage = "angular.ffFormBuilderDirective.message.cycleDetected";
						messages["logicsCycleDetected"].toastId = "logicsCycleDetected";
						return hasInvalidSteps;
					}
				}
			}
		}
	}
	return hasInvalidSteps;
};

var emptyLogicsTest = function (form, messages) {
	var hasInvalidSteps = false;

	for (var i in form.steps) {
		for (var j in form.steps[i].inputs) {
			if (form.steps[i].inputs[j].logics !== undefined && form.steps[i].inputs[j].logics.length < 1) {
				hasInvalidSteps = true;
				messages["emptyLogics"] = {};
				messages["emptyLogics"].errorInfo = form.steps[i].label
				+ "|" + form.steps[i].inputs[j].name;
				messages["emptyLogics"].validationMessage = "angular.ffFormBuilderDirective.message.logicsEmpty";
				messages["emptyLogics"].toastId = "emptyLogics";
				return hasInvalidSteps;
			}
		}
	}
	return hasInvalidSteps;
};