"use strict";

var ff = {};

ff.Filters = {};

ff.RenderingDirectives = {};

ff.Serializers = {};

ff.ValidationDirectives = {};

ff.LogicDirectives = {};

ff.Factories = {};

ff.validationRules = {};

ff.logicRules = {};

ff.dataProvider = null;

ff.app = angular.module("formFactory", [ "i18n", "formFactory.validation", "formFactory.logic", "ngFileUpload", "checklist-model", "formFactory.commonUseFactory", "formFactory.templateResolver", "ui.bootstrap", "formFactory.dataFactory", "vcRecaptcha" ]);

(function() {
    "use strict";
    var input = function($compile, $log, $window, $timeout) {
        var directive = {
            restrict: "E",
            require: [ "^ffController" ],
            scope: {
                actionData: "=",
                resultData: "=",
                callbackDirectives: "=",
                allCompletedFlag: "=",
                displayTemplates: "=",
                callback: "&"
            },
            link: linkFunction
        };
        return directive;
        function linkFunction(scope, el, attr, ctrls) {
            var formName = ctrls[0].formName;
            var listenTo = "ff" + formName + "NotifyComplete";
            var timeout = 5e3;
            scope.$on("callbackDone", function(event, callbackName) {
                scope.handleDoneDirectives(callbackName);
            });
            $window.document.addEventListener(listenTo, function(e) {
                scope.handleDoneGlobalCallbacks(e.detail.callbackName);
            });
            scope.doneDirectives = [];
            scope.doneGlobalCallbacks = [];
            scope.handleDoneDirectives = function(callbackName) {
                if (!_.contains(scope.doneDirectives, callbackName)) {
                    scope.doneDirectives.push(callbackName);
                    scope.checkIfAllCallbacksAreDone();
                }
            };
            scope.handleDoneGlobalCallbacks = function(callbackName) {
                if (!_.contains(scope.doneGlobalCallbacks, callbackName)) {
                    scope.doneGlobalCallbacks.push(callbackName);
                    scope.checkIfAllCallbacksAreDone();
                }
            };
            scope.checkIfAllCallbacksAreDone = function() {
                if (scope.doneDirectives.length === scope.callbackDirectives.length && scope.doneGlobalCallbacks.length === $window.ffCallbacks[formName].length) {
                    $timeout(function() {
                        scope.allCompletedFlag = true;
                        scope.runRedirectActions();
                    });
                    $log.info("All callbacks are done!!!");
                }
            };
            scope.executeGlobalCallbacks = function() {
                var data = {};
                data.actionData = scope.actionData;
                data.resultData = scope.resultData;
                for (var i in $window.ffCallbacks[formName]) {
                    var callbackObj = $window.ffCallbacks[formName][i];
                    callbackObj["fcn"].call(callbackObj["ctx"], data, callbackObj.formInfo);
                }
            };
            scope.runRedirectActions = function() {
                if (scope.callbackDirectives.length === 0 && $window.ffCallbacks[formName].length > 0) {
                    ctrls[0].handleRedirectIfPresent(scope.actionData);
                }
            };
            scope.executeGlobalCallbacks();
            for (var i in scope.callbackDirectives) {
                if (scope.displayTemplates) {
                    el.append($compile("<" + scope.callbackDirectives[i] + " action-data='actionData'></" + scope.callbackDirectives[i] + ">")(scope));
                } else {
                    el.css("display", "none");
                    el.append($compile("<" + scope.callbackDirectives[i] + " action-data='actionData'></" + scope.callbackDirectives[i] + ">")(scope));
                }
            }
            if (scope.callbackDirectives.length === 0 && $window.ffCallbacks[formName].length > 0) {
                $timeout(function() {
                    scope.allCompletedFlag = true;
                    scope.runRedirectActions();
                }, timeout);
            }
        }
    };
    angular.module("formFactory").directive("ffCallback", [ "$compile", "$log", "$window", "$timeout", input ]);
})();

ff.RenderingDirectives.FFFormController = function($scope, $log, _, contextualData, $http, Upload, ffFormDataFactory, ffCommonUseFactory, ffDataFactory, i18n, $window) {
    var vm = this;
    var deregisterFormControllerWatch = $scope.$watch(function() {
        return $scope[vm.formName];
    }, function(newValue) {
        if (!_.isUndefined(newValue)) {
            deregisterFormControllerWatch();
            $scope.$watch(function() {
                return $scope[vm.formName].$pristine;
            }, function(newValue) {
                vm.formStepStatus[vm.currentStep].touched = !newValue;
                if (vm.currentForm.steps[vm.currentStep].resetDisabled !== null) {
                    vm.currentForm.steps[vm.currentStep].resetDisabled = newValue;
                }
            });
        }
    });
    activate();
    function activate() {
        vm.formId = $scope.vm.id.split("_")[0];
        return getFormData().then(function() {
            $log.info("FFFormController is activated");
            vm.activated = true;
        });
    }
    function getFormData() {
        return ffFormDataFactory.getFormData($scope.vm.locale, vm.formId, contextualData.isPreviewMode).then(function(data) {
            for (var i in data.steps) {
                if (i == 0) {
                    data.steps[i].resetDisabled = true;
                } else {
                    data.steps[i].resetDisabled = null;
                }
            }
            vm.currentForm = data;
            vm.originalForm = angular.copy(data);
            vm.displaySubmissionText = false;
            vm.runCallbacks = false;
            vm.formName = "ff" + data.formName.replace(/[\-]/g, "");
            vm.currentStep = 0;
            vm.results = {};
            vm.update = update;
            vm.manageFormState = manageFormState;
            vm.captureInputsState = captureInputsState;
            vm.setInputsStatus = setInputsStatus;
            vm.getFormController = getFormController;
            vm.reset = reset;
            vm.submit = submit;
            vm.preventSubmit = preventSubmit;
            vm.handleSubmission = handleSubmission;
            vm.handleRedirectIfPresent = handleRedirectIfPresent;
            vm.saveForm = saveForm;
            vm.containsFile = false;
            vm.isSubmitted = isSubmitted;
            vm.ffTestsInitMethod = ffTestsInitMethod;
            vm.displayInputs = displayInputs;
            vm.populateAndSubmitResults = populateAndSubmitResults;
            vm.uploads = {};
            vm.copyInputs = copyInputs;
            vm.getPreviousStepInputValue = getPreviousStepInputValue;
            vm.getCurrentStepInputValue = getCurrentStepInputValue;
            vm.switchForm = switchForm;
            vm.resolveLogic = resolveLogic;
            vm.resolvePlaceholder = resolvePlaceholder;
            vm.resolveInputsize = resolveInputsize;
            vm.resolveName = resolveName;
            vm.resolveLabel = resolveLabel;
            vm.parseDate = parseDate;
            vm.isPreviewMode = contextualData.isPreviewMode;
            vm.isLiveMode = isLiveMode;
            vm.getCaptchaKey = getCaptchaKey;
            vm.getUniqueId = getUniqueId;
            vm.captchaLoaded = false;
            vm.notifyOfCaptchaLoad = notifyOfCaptchaLoad;
            vm.setUpCallbackSystem = setUpCallbackSystem;
            if (!_.isUndefined(vm.currentForm.layoutJson) && !_.isEmpty(vm.currentForm.layoutJson)) {
                vm.currentForm.layoutJson = JSON.parse(vm.currentForm.layoutJson);
                vm.isLayoutAvailable = vm.currentForm.layoutJson.active;
            } else {
                vm.isLayoutAvailable = false;
            }
            vm.prepareInputsForLayout = prepareInputsForLayout;
            vm.cssClassName = contextualData.cssClassName;
            vm.reset();
            init();
            vm.setUpCallbackSystem();
        });
    }
    function setUpCallbackSystem() {
        angular.element(document).ready(function() {
            var formInfo = {
                formName: vm.formName,
                notifyFormFactoryOfCompletion: function(callbackName, data) {
                    if (callbackName === undefined || callbackName === "" || callbackName === null) {
                        console.error("No callbackName specified! You must specify a callback name.");
                        return;
                    }
                    var event;
                    var callbackResult = {};
                    callbackResult.callbackName = callbackName;
                    callbackResult.data = data;
                    if (isMicrosoftBrowser()) {
                        event = $window.document.createEvent("CustomEvent");
                        event.initCustomEvent("ff" + this.formName + "NotifyComplete", false, false, callbackResult);
                    } else {
                        event = new CustomEvent("ff" + this.formName + "NotifyComplete", {
                            detail: callbackResult
                        });
                    }
                    document.dispatchEvent(event);
                },
                registerCallback: function(obj) {
                    if (obj.fcn === undefined || !(typeof obj.fcn === "function")) {
                        console.error("Improper callback object supplied. Read documentation!!!");
                    } else {
                        if (!callbackRegistered(obj)) {
                            window.ffCallbacks[this.formName].push(obj);
                        }
                    }
                }
            };
            var formReadyEvent;
            if (isMicrosoftBrowser()) {
                formReadyEvent = $window.document.createEvent("CustomEvent");
                formReadyEvent.initCustomEvent("ffFormReady", false, false, formInfo);
            } else {
                formReadyEvent = new CustomEvent("ffFormReady", {
                    detail: formInfo
                });
            }
            if ($window.ffCallbacks === undefined) {
                $window.ffCallbacks = {};
            }
            if ($window.ffCallbacks.registerCallback === undefined) {
                $window.ffCallbacks.registerCallback = function(formInfoObject, callbackRef, context) {
                    var obj = {
                        fcn: callbackRef,
                        ctx: context,
                        formInfo: formInfoObject
                    };
                    formInfoObject.registerCallback(obj);
                };
            }
            $window.ffCallbacks[vm.formName] = [];
            $window.document.dispatchEvent(formReadyEvent);
        });
        function callbackRegistered(obj) {
            for (var i in window.ffCallbacks[vm.formName]) {
                if (window.ffCallbacks[vm.formName][i].fcn === obj.fcn) {
                    return true;
                }
            }
            return false;
        }
    }
    function init() {
        vm.formStepStatus = {};
        vm.inputsStatus = {
            isDirty: {},
            ffVisible: {}
        };
        _.each(vm.currentForm.steps, function(step, index) {
            vm.formStepStatus[index] = {
                touched: false
            };
            _.each(step.inputs, function(input) {
                vm.inputsStatus.isDirty[input.name] = false;
                vm.inputsStatus.ffVisible[input.name] = false;
            });
        });
    }
    function notifyOfCaptchaLoad() {
        vm.captchaLoaded = true;
    }
    function getCaptchaKey() {
        if (contextualData.captchaApiKey !== undefined && contextualData.captchaApiKey !== "") {
            return contextualData.captchaApiKey;
        } else {
            return null;
        }
    }
    function getUniqueId(length) {
        var chars = "ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz".split("");
        if (!length) {
            length = Math.floor(Math.random() * chars.length);
        }
        var str = "";
        for (var i = 0; i < length; i++) {
            str += chars[Math.floor(Math.random() * chars.length)];
        }
        return str;
    }
    function switchForm() {
        ff.dataProvider = vm.currentTestFile.filePath;
        getFormData();
        $scope.$watch(function() {
            return !_.isUndefined($scope[vm.formName]);
        }, function(newValue) {
            if (newValue) {
                vm.reset();
            }
        });
    }
    function collectTestData() {
        $http.get(contextualData.context + "/modules/formfactory/live/jsonformfiles").then(function(data) {
            vm.testFiles = data.data.files;
            vm.currentTestFile = null;
        });
    }
    function submit() {
        var form = $scope[vm.formName];
        if (form.$valid) {
            if (form.$submitted) {
                $log.info("form already submitted");
            } else {
                vm.handleSubmission();
                form.$setSubmitted(true);
            }
        }
    }
    function preventSubmit(e) {
        e.preventDefault();
        $scope[vm.formName].$submitted = false;
    }
    function reset() {
        if (!_.isUndefined($scope[vm.formName])) {
            $scope[vm.formName].$setPristine();
            $scope[vm.formName].$setUntouched();
        }
        vm.inputs = angular.copy(vm.originalForm.steps[vm.currentStep].inputs);
        if (vm.isLayoutAvailable) {
            vm.currentForm.layoutJson.steps[vm.currentStep].rows = angular.copy(vm.currentForm.layoutJson.steps[vm.currentStep].rows);
            vm.prepareInputsForLayout();
        }
    }
    function update(advance) {
        var form = $scope[vm.formName];
        vm.copyInputs();
        if (vm.currentStep < vm.currentForm.steps.length) {
            vm.captureInputsState();
        }
        if (advance && form.$valid) {
            vm.currentStep++;
        } else if (!advance) {
            if (vm.currentStep !== 0) {
                vm.currentStep--;
            }
        }
        if (vm.currentStep < vm.currentForm.steps.length) {
            vm.setInputsStatus();
        }
        vm.inputs = [];
        if (vm.currentStep < vm.currentForm.steps.length) {
            vm.manageFormState();
            vm.displayInputs();
        } else if (!vm.isPreviewMode) {
            vm.populateAndSubmitResults();
        }
    }
    function copyInputs() {
        vm.currentForm.steps[vm.currentStep].inputs = [];
        _.forEach(vm.inputs, function(input) {
            if (input.value instanceof FileList) {
                vm.containsFile = true;
                var fileList = input.value;
                input.value = null;
                var cloned = _.clone(input);
                cloned.value = fileList;
                vm.currentForm.steps[vm.currentStep].inputs.push(cloned);
            } else {
                vm.currentForm.steps[vm.currentStep].inputs.push(_.clone(input));
            }
        });
    }
    function displayInputs() {
        _.forEach(vm.currentForm.steps[vm.currentStep].inputs, function(input) {
            if (input.value instanceof FileList) {
                var fileList = input.value;
                input.value = null;
                var cloned = _.clone(input);
                cloned.value = fileList;
                vm.inputs.push(cloned);
            } else {
                vm.inputs.push(_.clone(input));
            }
        });
        if (vm.isLayoutAvailable) {
            vm.prepareInputsForLayout();
        }
    }
    function populateAndSubmitResults() {
        _.forEach(vm.currentForm.steps, function(step) {
            _.forEach(step.inputs, function(input) {
                if (!_.isUndefined(input.value) && (!_.isEmpty(input.value) || _.isNumber(input.value) || _.isBoolean(input.value) || moment(input.value).isValid())) {
                    vm.results[input.name] = input.value;
                }
            });
        });
        vm.results["token"] = vm.currentForm.token;
        vm.submit();
    }
    function handleSubmission() {
        var fileCounter = 0;
        var fileUploadResponses = {};
        var fileLists = {};
        if (vm.containsFile) {
            _.each(vm.results, function(value, key) {
                if (value instanceof FileList) {
                    fileCounter++;
                    fileLists[key] = value;
                }
            });
            _.each(fileLists, function(list, key) {
                var fileType = !_.isEmpty(list[0].type) ? encodeURI(list[0].type.replace("/", "_")) : "application_octet-stream";
                Upload.http({
                    url: contextualData.context + "/modules/formfactory/live/fileupload/" + vm.formId + "/" + key + "/" + encodeURI(list[0].name + "/" + fileType),
                    headers: {
                        "Content-Type": "application/octet-stream"
                    },
                    data: list[0]
                }).then(function(result) {
                    fileCounter--;
                    fileUploadResponses[key] = [ result.data.data.tempFileName, result.data.data.originalFileName, result.data.data.type ];
                    if (fileCounter === 0) {
                        vm.saveForm(fileUploadResponses);
                    }
                }, function(result) {
                    $log.error(result);
                }, function(progress) {
                    vm.uploads[progress.config.data.name] = progress.loaded / progress.total * 100 + "%";
                });
            });
        } else {
            vm.saveForm();
        }
    }
    function saveForm(uploadResponses) {
        if (uploadResponses) {
            _.each(uploadResponses, function(key, value) {
                vm.results[value] = key;
            });
        }
        var req = {
            method: "POST",
            url: contextualData.context + "/modules/formfactory/live/" + vm.formId + "/" + $scope.vm.locale,
            data: vm.results
        };
        $http(req).success(function(data) {
            setSubmissionText("ff.message.submission");
            buildCallbackObjectIfNeedBe();
            if (vm.currentForm.callbacks !== null) {
                vm.actionData = data;
                vm.resultData = getResultData();
                vm.runCallbacks = true;
                return;
            }
            if (!handleRedirectIfPresent(data)) {
                vm.displaySubmissionText = true;
            }
            $log.info("Form successfully saved!!!!!");
        }).error(function(error) {
            vm.currentForm.afterSubmissionText = i18n.message("ff.message.submissionError");
            vm.displaySubmissionText = true;
            $log.info("form was not submitted due to ERROR!!!!");
        });
        function setSubmissionText(label) {
            if (vm.currentForm.afterSubmissionText === null || vm.currentForm.afterSubmissionText === "") {
                vm.currentForm.afterSubmissionText = i18n.message(label);
            }
        }
    }
    function getResultData() {
        var data = {};
        _.each(vm.results, function(value, key) {
            if (key !== "token") {
                data[key] = value;
            }
        });
        return data;
    }
    function buildCallbackObjectIfNeedBe() {
        if (vm.currentForm.callbacks === null && $window.ffCallbacks[vm.formName].length > 0) {
            vm.currentForm.callbacks = {
                callbacks: [],
                displayTemplates: false
            };
        }
    }
    function handleRedirectIfPresent(data) {
        var redirectAction = {};
        redirectAction.name = "";
        for (var i in data) {
            if ("actionName" in data[i] && (data[i].actionName[0] === "redirectToAPage" || data[i].actionName[0] === "redirectToUrl")) {
                redirectAction.url = data[i].redirectUrl[0];
                redirectAction.name = data[i].actionName[0];
            }
        }
        if (redirectAction.name === "redirectToAPage") {
            window.location.assign(redirectAction.url);
            return true;
        }
        if (redirectAction.name === "redirectToUrl") {
            window.location.assign(redirectAction.url);
            return true;
        }
        return false;
    }
    function getFormController() {
        return $scope[vm.formName];
    }
    function isSubmitted() {
        return vm.currentStep == vm.currentForm.steps.length;
    }
    function manageFormState() {
        if (vm.currentForm.steps[vm.currentStep].resetDisabled === null) {
            vm.getFormController().$setPristine();
            vm.currentForm.steps[vm.currentStep].resetDisabled = true;
        } else {
            if (vm.currentForm.steps[vm.currentStep].resetDisabled) {
                vm.getFormController().$setPristine();
            } else {
                vm.getFormController().$setDirty();
            }
        }
    }
    function ffTestsInitMethod(obj) {
        vm.currentForm = obj;
        vm.originalForm = angular.copy(obj);
        vm.formName = obj.formName;
        vm.reset();
        $scope.$digest();
        return true;
    }
    function getPreviousStepInputValue(inputId) {
        var searchStep = vm.currentStep - 1;
        if (vm.currentStep > 0 && searchStep >= 0) {
            for (searchStep; searchStep >= 0; searchStep--) {
                var searchedInput = _.find(vm.currentForm.steps[searchStep].inputs, function(input) {
                    if (input.name === inputId) {
                        return true;
                    }
                });
                if (!_.isEmpty(searchedInput)) {
                    return {
                        value: searchedInput.value,
                        validity: null
                    };
                }
            }
        }
        return undefined;
    }
    function getCurrentStepInputValue(inputId) {
        var formController = vm.getFormController();
        var searchedInput = _.find(vm.inputs, function(input) {
            if (input.name === inputId) {
                return true;
            }
        });
        if (!_.isEmpty(searchedInput)) {
            return {
                value: searchedInput.value,
                validity: formController[searchedInput.name].$valid
            };
        }
        return null;
    }
    function captureInputsState() {
        var formController = vm.getFormController();
        _.each(vm.currentForm.steps[vm.currentStep].inputs, function(input) {
            if (formController[input.name] !== undefined) {
                vm.inputsStatus.isDirty[input.name] = formController[input.name].$dirty;
                vm.inputsStatus.ffVisible[input.name] = formController[input.name].ffVisible;
            }
        });
    }
    function setInputsStatus() {
        var formController = vm.getFormController();
        _.each(vm.currentForm.steps[vm.currentStep].inputs, function(input) {
            if (vm.inputsStatus.isDirty[input.name]) {
                if (_.isUndefined(formController[input.name])) {
                    var deregisterInputWatch = $scope.$watch(function() {
                        return !_.isUndefined(formController[input.name]);
                    }, function(newValue) {
                        if (newValue) {
                            deregisterInputWatch();
                            formController[input.name].$setDirty();
                            formController[input.name].ffVisible = vm.inputsStatus.ffVisible[input.name];
                        }
                    });
                } else {
                    formController[input.name].$setDirty();
                    formController[input.name].ffVisible = vm.inputsStatus.ffVisible[input.name];
                }
            }
        });
    }
    function resolveLogic(input, form) {
        return ffCommonUseFactory.logicResolver(input, form);
    }
    function resolvePlaceholder(input) {
        return ffCommonUseFactory.placeholderResolver(input);
    }
    function resolveInputsize(input) {
        return ffCommonUseFactory.inputsizeResolver(input);
    }
    function resolveName(input) {
        return ffCommonUseFactory.nameResolver(input);
    }
    function resolveLabel(input) {
        return ffCommonUseFactory.labelResolver(input);
    }
    function parseDate(value) {
        return ffCommonUseFactory.parseDate(value);
    }
    function isLiveMode() {
        return true;
    }
    function prepareInputsForLayout() {
        vm.preparedLayoutInputs = {};
        _.each(vm.inputs, function(input, index) {
            vm.preparedLayoutInputs[input.layoutUUID] = index;
        });
    }
    function isMicrosoftBrowser() {
        return $window.navigator.appName == "Microsoft Internet Explorer" || $window.navigator.appName == "Netscape" && new RegExp("Trident/.*rv:([0-9]{1,}[.0-9]{0,})").exec($window.navigator.userAgent) != null;
    }
};

ff.RenderingDirectives.FFFormController.$inject = [ "$scope", "$log", "_", "contextualData", "$http", "Upload", "ffFormDataFactory", "ffCommonUseFactory", "ffDataFactory", "i18nService", "$window" ];

(function() {
    "use strict";
    ff.Factories.formDataFactory = function($http, $log, contextualData) {
        return {
            getFormData: getFormData
        };
        function getFormData(locale, formId, inPreviewMode) {
            if (ff.dataProvider === null) {
                if (inPreviewMode !== "") {
                    return $http.get(contextualData.context + "/modules/formfactory/live/preview/" + formId + "/" + locale).then(getFormDataComplete).catch(getFormDataFailed);
                } else {
                    return $http.get(contextualData.context + "/modules/formfactory/live/" + formId + "/" + locale).then(getFormDataComplete).catch(getFormDataFailed);
                }
            }
            function getFormDataComplete(response) {
                $log.debug("ffFormDataFactory completed data transfer!");
                $log.debug(response);
                return response.data;
            }
            function getFormDataFailed(error) {
                $log.debug("XHR Failed for getFormData." + error.data);
            }
        }
    };
    ff.Factories.formDataFactory.$inject = [ "$http", "$log", "contextualData" ];
    angular.module("formFactory").factory("ffFormDataFactory", ff.Factories.formDataFactory);
})();

(function() {
    "use strict";
    ff.RenderingDirectives.renderInput = function($compile, $log) {
        var directive = {
            restrict: "E",
            require: [ "^ffController" ],
            scope: {
                input: "=",
                resolveLogic: "&",
                resolvePlaceholder: "&",
                resolveInputsize: "&",
                resolveName: "&",
                resolveLabel: "&"
            },
            link: postLinkFunc
        };
        return directive;
        function postLinkFunc(scope, el, attr, ffControllerCtrl) {
            var formController = ffControllerCtrl[0];
            scope.form = formController.getFormController();
            scope.readOnly = !formController.isLiveMode();
            el.append($compile(angular.element(scope.input.template).attr("view-type", !_.isUndefined(formController.currentForm.theme) && formController.currentForm.theme !== null ? formController.currentForm.theme : ""))(scope));
            scope.isRequired = function isRequired() {
                var inputModel = scope.form[scope.input.name];
                var required = _.isUndefined(scope.input.validations) ? false : !_.isUndefined(scope.input.validations.required) || !_.isUndefined(scope.input.validations["matrix-required"]);
                return required && _.isUndefined(inputModel) || required && inputModel.ffVisible;
            };
            scope.checkValues = function checkValues() {
                var model = scope.form[scope.input.name];
                var required = _.isUndefined(scope.input.validations) ? false : !_.isUndefined(scope.input.validations.required);
                if (required && _.isUndefined(model)) {
                    return _.find(scope.input.value, function(value) {
                        return value;
                    }) == undefined;
                } else if (required && model.ffVisible) {
                    return _.find(scope.input.value, function(value) {
                        return value;
                    }) == undefined && model.ffVisible;
                }
            };
            scope.makeDirty = function makeDirty() {
                var model = scope.form[scope.input.name];
                model.$dirty = true;
            };
        }
    };
    angular.module("formFactory").directive("ffInput", [ "$compile", "$log", ff.RenderingDirectives.renderInput ]);
})();

(function() {
    "use strict";
    ff.LogicDirectives.applyLogicRulesInput = function($log, _, LogicService, $timeout) {
        var directive = {
            restrict: "A",
            require: [ "^ffController", "?ngModel" ],
            scope: false,
            link: linkFunc
        };
        return directive;
        function linkFunc(scope, el, attr, ctrls) {
            var model = ctrls[1];
            var input = scope.input;
            if (!_.isUndefined(model) && model !== null) {
                var listener = function(newValue, oldValue, scope) {
                    var val = newValue;
                    newValue = val.value;
                    var fieldName = val.fieldName;
                    $timeout(function() {
                        if (_.isUndefined(newValue)) {
                            model.ffVisible = false;
                            return;
                        }
                        var isVisible = true;
                        _.each(model.$logics, function(logicRule, ruleFieldReference) {
                            if (ruleFieldReference.split(":")[1] === fieldName) {
                                if (_.isFunction(logicRule)) {
                                    LogicService.$logicRulesValues[model.$name][ruleFieldReference] = logicRule(scope, newValue);
                                } else {
                                    $log.error("Trying to call a logic rule that is not a function");
                                    $log.error(logicRule);
                                    isVisible = false;
                                }
                            }
                        });
                        var rulesAggregateValue = true;
                        if (isVisible) {
                            _.each(LogicService.$logicRulesValues[model.$name], function(value) {
                                rulesAggregateValue = rulesAggregateValue && value;
                            });
                            isVisible = rulesAggregateValue;
                        }
                        model.ffVisible = isVisible;
                        if (model.ffVisible) {
                            $timeout(function() {
                                model.$validate();
                            });
                        }
                    });
                };
                model.ffVisible = _.isUndefined(input.logics);
                model.$logics = {};
                watchVisibilityToggle(scope, input);
                angular.forEach(scope.input.logics, function(rule) {
                    var ruleName = getRuleName(rule.name);
                    var formController = ctrls[0].getFormController();
                    var searchedInputValue = ctrls[0].getPreviousStepInputValue(rule.fieldName);
                    if (_.isUndefined(searchedInputValue) && _.isUndefined(formController[rule.fieldName])) {
                        var input = scope.input;
                        var deregisterWatch = scope.$watch(function() {
                            return _.isUndefined(formController[rule.fieldName]);
                        }, function() {
                            if (!_.isUndefined(formController[rule.fieldName])) {
                                angular.forEach(input.logics, function(rule) {
                                    var ruleName = getRuleName(rule.name);
                                    ctrls[1] = model;
                                    LogicService.initLogicRule(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                                });
                                watchRuleFieldValueToApplyLogic(formController, rule, listener);
                                deregisterWatch();
                            }
                        });
                    } else {
                        LogicService.initLogicRule(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                        watchRuleFieldValueToApplyLogic(formController, rule, listener);
                    }
                });
                if (_.isUndefined(LogicService.$logicRulesValues[model.$name])) {
                    LogicService.$logicRulesValues[model.$name] = {};
                    _.each(model.$logics, function(logicRule, ruleFieldReference) {
                        LogicService.$logicRulesValues[model.$name][ruleFieldReference] = false;
                    });
                }
            } else {
                $log.debug("There is no model for this input " + scope.input.name);
                var formController = ctrls[0].getFormController();
                var deregisterWatch = scope.$watch(function() {
                    return _.isUndefined(formController[input.name]);
                }, function() {
                    if (!_.isUndefined(formController[input.name])) {
                        var inputModelController = formController[input.name];
                        inputModelController.ffVisible = _.isUndefined(input.logics);
                        inputModelController.$logics = {};
                        watchVisibilityToggle(scope, input);
                        var listener = function(newValue, oldValue, scope) {
                            var val = newValue;
                            newValue = val.value;
                            var fieldName = val.fieldName;
                            $timeout(function() {
                                if (_.isUndefined(newValue)) {
                                    inputModelController.ffVisible = false;
                                    return;
                                }
                                var isVisible = true;
                                _.each(inputModelController.$logics, function(logicRule, ruleFieldReference) {
                                    if (ruleFieldReference.split(":")[1] === fieldName) {
                                        if (_.isFunction(logicRule)) {
                                            LogicService.$logicRulesValues[inputModelController.$name][ruleFieldReference] = logicRule(scope, newValue);
                                        } else {
                                            $log.error("Trying to call a logic rule that is not a function");
                                            $log.error(logicRule);
                                            isVisible = false;
                                        }
                                    }
                                });
                                var rulesAggregateValue = true;
                                if (isVisible) {
                                    _.each(LogicService.$logicRulesValues[inputModelController.$name], function(value) {
                                        rulesAggregateValue = rulesAggregateValue && value;
                                    });
                                    isVisible = rulesAggregateValue;
                                }
                                inputModelController.ffVisible = isVisible;
                                if (inputModelController.ffVisible) {
                                    $timeout(function() {
                                        inputModelController.$validate();
                                    });
                                }
                            });
                        };
                        angular.forEach(input.logics, function(rule) {
                            var ruleName = getRuleName(rule.name);
                            ctrls[1] = inputModelController;
                            LogicService.initLogicRule(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                            watchRuleFieldValueToApplyLogic(formController, rule, listener);
                        });
                        deregisterWatch();
                        LogicService.$logicRulesValues[inputModelController.$name] = {};
                        _.each(inputModelController.$logics, function(logicRule, ruleFieldReference) {
                            LogicService.$logicRulesValues[inputModelController.$name][ruleFieldReference] = false;
                        });
                    }
                });
            }
            function watchVisibilityToggle(scope, input) {
                scope.$watch(function(scope) {
                    return scope.form[input.name].ffVisible;
                }, function(newValue, oldValue) {
                    if (!_.isUndefined(newValue) && !newValue && scope.form[input.name].$dirty) {
                        if (_.isArray(input.value)) {
                            input.value = [];
                        } else if (_.isObject(input.value)) {
                            input.value = {};
                        } else {
                            input.value = null;
                        }
                    }
                    if (!_.isUndefined(newValue) && !newValue) {
                        $timeout(function() {
                            _.each(scope.form[input.name].$validators, function(validation, validationName) {
                                scope.form[input.name].$setValidity(validationName, true);
                            });
                        });
                    }
                });
            }
            function watchRuleFieldValueToApplyLogic(formController, rule, listener) {
                if (!_.isUndefined(formController[rule.fieldName])) {
                    registerValueWatches(ctrls, rule, listener);
                } else {
                    var result = ctrls[0].getPreviousStepInputValue(rule.fieldName);
                    if (_.isUndefined(result)) {
                        result = {};
                        result.value = undefined;
                        result.validity = undefined;
                    }
                    var previousStepInputValue = {
                        value: result.value,
                        validity: result.validity,
                        fieldName: rule.fieldName
                    };
                    if (!_.isUndefined(previousStepInputValue.value)) {
                        listener(previousStepInputValue);
                    } else {
                        var unregisterFieldNameWatch = scope.$watch(function() {
                            return !_.isUndefined(formController[rule.fieldName]);
                        }, function(newValue) {
                            if (newValue) {
                                registerValueWatches(ctrls, rule, listener);
                                unregisterFieldNameWatch();
                            }
                        });
                    }
                }
            }
            function registerValueWatches(ctrls, rule, listener) {
                var inputValue = ctrls[0].getCurrentStepInputValue(rule.fieldName);
                if (_.isUndefined(inputValue) || _.isNull(inputValue)) {
                    scope.$watch(function(scope) {
                        var result = ctrls[0].getCurrentStepInputValue(rule.fieldName);
                        return {
                            value: result.value,
                            validity: result.validity,
                            fieldName: rule.fieldName
                        };
                    }, listener, true);
                } else {
                    scope.$watch(function(scope) {
                        var result = ctrls[0].getCurrentStepInputValue(rule.fieldName);
                        return {
                            value: result.value,
                            validity: result.validity,
                            fieldName: rule.fieldName
                        };
                    }, listener, true);
                }
            }
            function getRuleName(ruleType) {
                return ruleType.split("_")[0];
            }
        }
    };
    angular.module("formFactory").directive("ffLogic", [ "$log", "_", "LogicService", "$timeout", ff.LogicDirectives.applyLogicRulesInput ]);
})();

(function() {
    "use strict";
    ff.RenderingDirectives.parseJSON = function($log, ValidationService) {
        var directive = {
            restrict: "A",
            require: [ "^ffController", "?ngModel" ],
            scope: false,
            link: postLinkFunc
        };
        return directive;
        function postLinkFunc(scope, el, attr, ctrls) {
            var model = ctrls[1];
            var parser = function(inputValue) {
                if (_.isString(inputValue)) {
                    var transformedInput = JSON.parse(inputValue);
                    model.$setViewValue(transformedInput);
                    return transformedInput;
                }
                return inputValue;
            };
            if (!_.isUndefined(model) && model != null) {
                model.$parsers.push(parser);
            } else {
                var formController = ctrls[0].getFormController();
                var input = scope.input;
                var deregisterWatch = scope.$watchCollection(formController, function() {
                    if (!_.isUndefined(formController[input.name])) {
                        var model = formController[input.name];
                        model.$parsers.push(parser);
                        deregisterWatch();
                    }
                });
            }
        }
    };
    angular.module("formFactory").directive("ffParseJson", [ "$log", ff.RenderingDirectives.parseJSON ]);
})();

(function() {
    "use strict";
    ff.ValidationDirectives.validateInput = function($log, ValidationService) {
        var directive = {
            restrict: "A",
            require: [ "^ffController", "?ngModel" ],
            scope: false,
            link: postLinkFunc
        };
        return directive;
        function postLinkFunc(scope, el, attr, ctrls) {
            var model = ctrls[1];
            var input = scope.input;
            if (!_.isUndefined(model) && model != null) {
                angular.forEach(scope.input.validations, function(rule, ruleName) {
                    if (ruleName != "required") {
                        ValidationService.initValidator(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                    }
                });
            } else {
                var formController = ctrls[0].getFormController();
                var deregisterWatch = scope.$watchCollection(formController, function() {
                    if (!_.isUndefined(formController[input.name])) {
                        angular.forEach(scope.input.validations, function(rule, ruleName) {
                            console.log("rule: " + ruleName);
                            if (ruleName != "required") {
                                ctrls[1] = formController[input.name];
                                ValidationService.initValidator(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                            }
                        });
                        deregisterWatch();
                    }
                });
            }
            scope.ffValidate = function() {
                scope.form[input.name].$validate();
            };
        }
    };
    angular.module("formFactory").directive("ffValidations", [ "$log", "ValidationService", ff.ValidationDirectives.validateInput ]);
})();

(function() {
    "use strict";
    ff.registerLogicRules = function(logicRules) {
        angular.module("formFactory").config([ "LogicServiceProvider", function(LogicServiceProvider) {
            angular.forEach(logicRules, function(value, key) {
                LogicServiceProvider.registerLogicRule(key, value);
            });
        } ]);
    };
    ff._LogicServiceProvider = function() {
        var logicRules = {};
        this.registerLogicRule = function(name, fct) {
            logicRules[name] = fct;
        };
        this.$get = [ function() {
            return new ff._LogicService(logicRules);
        } ];
    };
    ff._LogicService = function(logicRules) {
        this.$logicRulesValues = {};
        this.initLogicRule = function(name, rule, scope, el, attr, ctrls) {
            if (_.isFunction(logicRules[name])) {
                ctrls[1].$logics[name + ":" + rule.fieldName] = logicRules[name](rule, scope, el, attr, ctrls);
            }
        };
    };
    angular.module("formFactory.logic", []).provider("LogicService", ff._LogicServiceProvider);
    ff.registerLogicRules(ff.logicRules);
})();

(function() {
    "use strict";
    ff.registerValidators = function(validationRules) {
        angular.module("formFactory").config([ "ValidationServiceProvider", function(ValidationServiceProvider) {
            angular.forEach(validationRules, function(value, key) {
                ValidationServiceProvider.registerValidator(key, value);
            });
        } ]);
    };
    ff._ValidationServiceProvider = function() {
        var validators = {};
        this.registerValidator = function(name, fct) {
            validators[name] = fct;
        };
        this.$get = [ function() {
            return new ff._ValidationService(validators);
        } ];
    };
    ff._ValidationService = function(validators) {
        this.initValidator = function(name, rule, scope, el, attr, ctrls, $parse) {
            if (validators[name] && _.isUndefined(ctrls[1].$validators[name])) {
                var validationRule = validators[name](rule, scope, el, attr, ctrls, $parse);
                if (_.isFunction(validationRule)) {
                    ctrls[1].$validators[name] = validationRule;
                }
            }
        };
    };
    angular.module("formFactory.validation", []).provider("ValidationService", ff._ValidationServiceProvider);
    ff.registerValidators(ff.validationRules);
})();

(function() {
    var CommonUseFactory = function($http, contextualData, $log) {
        return {
            checkIfNameExists: checkFormAvailability,
            logicResolver: logicResolver,
            valueResolver: valueResolver,
            placeholderResolver: placeholderResolver,
            inputsizeResolver: inputsizeResolver,
            nameResolver: nameResolver,
            labelResolver: labelResolver,
            parseDate: parseDate,
            createMoment: createMoment
        };
        function checkFormAvailability(newName, flag, formIdentifier) {
            var self = this;
            if (!_.isUndefined(newName) && newName != "") {
                var data = {
                    formName: newName,
                    siteIdentifier: contextualData.siteIdentifier
                };
                if (!_.isUndefined(formIdentifier)) {
                    data.formIdentifier = formIdentifier;
                }
                $http({
                    url: contextualData.context + "/modules/formfactory/builder/checkformnameavailability/" + contextualData.locale,
                    method: "POST",
                    data: data
                }).then(function(response) {
                    self[flag] = response.data.available;
                });
            } else {
                self[flag] = false;
            }
        }
        function valueResolver(args) {
            console.log("Resolving: ");
        }
        function placeholderResolver(input) {
            if ("placeholder" in input) {
                return input.placeholder;
            }
            return null;
        }
        function logicResolver(input, form) {
            if (input !== undefined) {
                if (!("ffVisible" in form[input.name])) {
                    return true;
                }
                return form[input.name].ffVisible;
            }
            return true;
        }
        function inputsizeResolver(input) {
            if ("inputsize" in input) {
                return input.inputsize;
            }
            return null;
        }
        function nameResolver(input) {
            if ("name" in input) {
                return input.name;
            }
            return null;
        }
        function labelResolver(input) {
            if ("label" in input) {
                return input.label;
            }
            return null;
        }
        function parseDate(value, format) {
            if (_.isUndefined(value) || value === null || value instanceof Date && isNaN(value.getTime())) {
                return false;
            }
            format = format != null ? format : "YYYY-MM-DD";
            if (value instanceof Date) {
                value = moment(value.toISOString()).format(format);
            } else if (moment.isMoment(value)) {
                return true;
            }
            if (typeof value === "string") {
                var components = value.split("-");
                if (components.length === 3) {
                    components = components[2].split(":");
                    if (components.length === 3) {
                        components = components[2].split(".");
                        if (components.length === 2) {
                            return true;
                        }
                    }
                }
            }
            return validateFormat(value, [ "-", "/" ], format).isValid;
        }
        function createMoment(value, formatSeparators, format) {
            if (_.isUndefined(value) || value === null) {
                return null;
            }
            if (value instanceof Date) {
                return moment(value.toISOString());
            } else if (moment(value, moment.ISO_8601, true).isValid()) {
                return moment(value, moment.ISO_8601, true).toISOString();
            }
            format = format != null ? format : "YYYY-MM-DD";
            var formatSeparators = _.isUndefined(formatSeparators) || formatSeparators === null ? [ "-", "/" ] : _.isString(formatSeparators) ? [].push(formatSeparators) : formatSeparators;
            var validatedDate = validateFormat(value, formatSeparators, format);
            return validatedDate.isValid ? validatedDate.date.toISOString() : null;
        }
        function validateFormat(value, formatSeparators, format) {
            var validatedDate = {
                isValid: false,
                date: null
            };
            for (var i in formatSeparators) {
                var sections = value.toString().split(formatSeparators[i]);
                var formatSections = format.split(formatSeparators[i]);
                if (sections.length == formatSections.length) {
                    validatedDate.isValid = true;
                    var formatSectionIndexes = {
                        year: null,
                        month: null,
                        date: null
                    };
                    for (var j in sections) {
                        if (sections[j] === undefined || sections[j] === "" || sections[j] === null || isNaN(sections[j]) || parseInt(sections[j]) < 0 || sections[j].toString().length !== formatSections[j].length) {
                            validatedDate.isValid = false;
                            break;
                        }
                        if (formatSections[j].toLowerCase().indexOf("y") !== -1) {
                            formatSectionIndexes.year = j;
                        } else if (formatSections[j].toLowerCase().indexOf("m") !== -1) {
                            formatSectionIndexes.month = j;
                        } else if (formatSections[j].toLowerCase().indexOf("d") !== -1) {
                            formatSectionIndexes.date = j;
                        }
                        sections[j] = parseInt(sections[j], 10);
                    }
                    if (!validatedDate.isValid) {
                        break;
                    }
                    validatedDate.date = moment();
                    if (!_.isNull(formatSectionIndexes.year)) {
                        validatedDate.date.year(sections[formatSectionIndexes.year]);
                    }
                    if (!_.isNull(formatSectionIndexes.month)) {
                        validatedDate.date.month(sections[formatSectionIndexes.month] - 1);
                    }
                    if (!_.isNull(formatSectionIndexes.date)) {
                        validatedDate.date.date(sections[formatSectionIndexes.date]);
                    }
                    var formatString = "";
                    for (var sec in formatSections) {
                        formatString += formatSections[sec].substring(0, 1);
                    }
                    validatedDate.isValid = validatedDate.date.format(formatString) == sections.join("-");
                }
            }
            return validatedDate;
        }
    };
    CommonUseFactory.$inject = [ "$http", "contextualData", "$log" ];
    angular.module("formFactory.commonUseFactory", []).factory("ffCommonUseFactory", CommonUseFactory);
})();

(function() {
    var dataFactory = function($http, $log) {
        return {
            getData: getData,
            customRequest: customRequest
        };
        function getData(url) {
            return $http.get(url).then(getDataComplete).catch(getDataFailed);
        }
        function customRequest(obj) {
            return $http(obj).then(getDataComplete);
        }
        function getDataComplete(response) {
            return response.data;
        }
        function getDataFailed(error) {
            $log.error("XHR Failed to execute your request." + error.data);
            return error.data;
        }
    };
    dataFactory.$inject = [ "$http", "$log" ];
    angular.module("formFactory.dataFactory", []).factory("ffDataFactory", dataFactory);
})();

(function() {
    var parseDate = function($log, ffCommonUseFactory, $timeout) {
        var directive = {
            restrict: "A",
            require: [ "^ngModel" ],
            link: linkFunction
        };
        return directive;
        function linkFunction(scope, el, attr, ctrls) {
            var model = ctrls[0];
            var hasParserExecuted = false;
            var parserInProgress = false;
            var viewValue;
            var AVAILABLE_FORMAT_SEPARATORS = [ "-", "/" ];
            var FORMAT_TEMPLATE = attr.format != null ? attr.format : "YYYY-MM-DD";
            var MAX_CHARACTERS = getMaxCharacters(FORMAT_TEMPLATE);
            var FORMAT_SEPARATOR_PROVIDED = !_.isUndefined(attr.ffParseDate) && attr.ffParseDate !== null && attr.ffParseDate.trim().length > 0;
            var REGEXP_FORMAT_SEPARATOR = FORMAT_SEPARATOR_PROVIDED ? attr.ffParseDate : "-|/";
            var FORMAT_REGEXP_PATTERN = "(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+(?=[0-9]*)" + "|(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "]" + "|(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+" + "|(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "]" + "|(^[0-9]+)";
            var FORMAT_REGEXP = RegExp(FORMAT_REGEXP_PATTERN, "g");
            var REPLACE_REGEXP = RegExp(REGEXP_FORMAT_SEPARATOR, "g");
            var initializeViewValue = scope.$watch(function() {
                return model.$viewValue;
            }, function(value) {
                if (hasParserExecuted) {
                    initializeViewValue();
                    return;
                }
                viewValue = value;
                initializeViewValue();
            });
            function getFormattedRepresentation(value, formatSeparator) {
                if (value instanceof Date) {
                    return moment(value.toISOString(), moment.ISO_8601, true).format(FORMAT_TEMPLATE).replace(/-/g, formatSeparator);
                } else if (typeof value === "string" && moment(value, moment.ISO_8601, true).isValid()) {
                    return moment(value, moment.ISO_8601, true).format(FORMAT_TEMPLATE).replace(/-/g, formatSeparator);
                } else {
                    return value;
                }
            }
            function getFormatSeparator(value) {
                if (FORMAT_SEPARATOR_PROVIDED) {
                    return REGEXP_FORMAT_SEPARATOR;
                } else {
                    var formatSeparator = "-";
                    if (!_.isUndefined(value) && value !== null && !(value instanceof Date)) {
                        var separatorIndex = -1;
                        for (var i in AVAILABLE_FORMAT_SEPARATORS) {
                            var currentSeparatorIndex = value.indexOf(AVAILABLE_FORMAT_SEPARATORS[i]);
                            if (currentSeparatorIndex !== -1 && (separatorIndex === -1 || currentSeparatorIndex < separatorIndex)) {
                                separatorIndex = currentSeparatorIndex;
                                formatSeparator = AVAILABLE_FORMAT_SEPARATORS[i];
                            }
                        }
                    }
                    return formatSeparator;
                }
            }
            function setViewValueFromDateOrMoment(value, separator) {
                var m = value instanceof Date ? moment(viewValue.toISOString(), moment.ISO_8601, true) : moment(viewValue, moment.ISO_8601, true);
                value = m.format(FORMAT_TEMPLATE);
                if (separator !== "-") {
                    value.replace(/-/g, separator);
                }
                model.$setViewValue(value);
                model.$render();
                return m.toISOString();
            }
            function getMaxCharacters(format) {
                var maxCharacters;
                for (var i in AVAILABLE_FORMAT_SEPARATORS) {
                    if (format.indexOf(AVAILABLE_FORMAT_SEPARATORS[i]) !== -1) {
                        var sections = format.split(AVAILABLE_FORMAT_SEPARATORS[i]);
                        maxCharacters = sections.length - 1;
                        for (var j in sections) {
                            maxCharacters += sections[j].length;
                        }
                        break;
                    }
                }
                return maxCharacters;
            }
            model.$parsers.push(function(value) {
                hasParserExecuted = true;
                if (parserInProgress) {
                    return value;
                }
                parserInProgress = true;
                if (_.isUndefined(value) || value === null) {
                    parserInProgress = false;
                    viewValue = value;
                    return value;
                }
                if (value instanceof Date) {
                    model.$setViewValue(getFormattedRepresentation(value, getFormatSeparator(viewValue)));
                    viewValue = value;
                    model.$render();
                    parserInProgress = false;
                    return moment(value.toISOString(), moment.ISO_8601, true).toISOString();
                }
                var matchedValue = value.match(FORMAT_REGEXP);
                if (matchedValue === null) {
                    value = viewValue;
                    model.$setViewValue(getFormattedRepresentation(value, getFormatSeparator(value)));
                    model.$render();
                    parserInProgress = false;
                    return value;
                }
                var formatSeparator = getFormatSeparator(value);
                value = viewValue !== null && matchedValue[0].length < viewValue.length && matchedValue[0].length != value.length ? viewValue : matchedValue[0];
                value = value.replace(REPLACE_REGEXP, formatSeparator);
                if (value.toString().length > MAX_CHARACTERS) {
                    if (ffCommonUseFactory.parseDate(value, FORMAT_TEMPLATE)) {
                        var momentFormat = formatSeparator !== "-" ? angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator) : FORMAT_TEMPLATE;
                        var m = moment(ffCommonUseFactory.createMoment(value, FORMAT_SEPARATOR_PROVIDED ? REGEXP_FORMAT_SEPARATOR : [ "-", "/" ], FORMAT_TEMPLATE), moment.ISO_8601, true);
                        value = m.toISOString();
                        viewValue = value;
                        model.$setViewValue(m.format(momentFormat));
                        model.$render();
                    } else {
                        if (viewValue instanceof Date || moment(viewValue, moment.ISO_8601, true).isValid()) {
                            value = setViewValueFromDateOrMoment(viewValue, formatSeparator);
                        } else {
                            value = viewValue;
                            model.$setViewValue(value);
                            model.$render();
                            value = moment(value, angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator), true).toISOString();
                        }
                    }
                } else {
                    viewValue = value;
                    model.$setViewValue(value);
                    model.$render();
                    if (ffCommonUseFactory.parseDate(value, FORMAT_TEMPLATE)) {
                        value = moment(value, angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator)).toISOString();
                    }
                }
                $timeout(function() {
                    model.$setValidity("date", true);
                });
                parserInProgress = false;
                return value;
            });
        }
    };
    angular.module("formFactory").directive("ffParseDate", [ "$log", "ffCommonUseFactory", "$timeout", parseDate ]);
})();

(function() {
    "use strict";
    var datePicker = function($log, contextualData) {
        var directive = {
            restrict: "E",
            scope: {
                format: "@?",
                minValue: "=?",
                maxValue: "=?",
                datePickerSize: "=",
                minDatePlaceholder: "=",
                maxDatePlaceholder: "=",
                datePickerOptions: "=?",
                datePickerMode: "@?",
                minLabel: "@?",
                maxLabel: "@?",
                readOnly: "@?",
                isDisabled: "@?",
                enforceMinValue: "@?",
                enforceMaxValue: "@?",
                errorControl: "=?",
                parserSeparator: "@?"
            },
            controller: DatePickerController,
            controllerAs: "dpc",
            bindToController: true,
            templateUrl: contextualData.context + "/modules/form-factory-core/javascript/angular/components/datePicker.html",
            link: linkFunc
        };
        return directive;
        function linkFunc(scope, el, attr, ctrl) {}
    };
    angular.module("formFactory").directive("ffDatePicker", [ "$log", "contextualData", datePicker ]);
    var DatePickerController = function($scope, $log, i18n, uibDateParser, $timeout, ffCommonUseFactory) {
        var dpc = this;
        var DEFAULT_FORMAT = "YYYY-MM-DD";
        dpc.isOpenMin = false;
        dpc.isOpenMax = false;
        dpc.previousValue = {
            min: "",
            max: ""
        };
        dpc.selectedMinDate;
        dpc.selectedMaxDate;
        dpc.openDatePicker = openDatePicker;
        dpc.executeOnChange = executeOnChange;
        dpc.i18nMessageGetter = i18n.message;
        dpc.format = dpc.format == null || dpc.format.trim().length == 0 ? DEFAULT_FORMAT : dpc.format;
        dpc.uibDateParserFormat = convertFormatToUibDateFormat(dpc.format);
        activate();
        function activate() {
            dpc.isAvailableDatepicker = {
                min: _.isUndefined(dpc.minValue) ? false : true,
                max: _.isUndefined(dpc.maxValue) ? false : true
            };
            if (_.isObject(dpc.errorControl)) {
                if (!_.isUndefined(dpc.errorControl.min) && _.isObject(dpc.errorControl.min)) {
                    dpc.errorControl.min.hasError = !_.isUndefined(dpc.errorControl.min.hasError) && _.isBoolean(dpc.errorControl.min.hasError) ? dpc.errorControl.min.hasError : false;
                } else {
                    dpc.errorControl.min = {
                        hasError: false,
                        errorMessage: null
                    };
                }
                if (!_.isUndefined(dpc.errorControl.max) && _.isObject(dpc.errorControl.max)) {
                    dpc.errorControl.max.hasError = !_.isUndefined(dpc.errorControl.max.hasError) && _.isBoolean(dpc.errorControl.max.hasError) ? dpc.errorControl.max.hasError : false;
                } else {
                    dpc.errorControl.max = {
                        hasError: false,
                        errorMessage: null
                    };
                }
            } else {
                dpc.errorControl = {
                    min: {
                        hasError: false,
                        errorMessage: null
                    },
                    max: {
                        hasError: false,
                        errorMessage: null
                    }
                };
            }
            dpc.minLabelAvailable = !_.isUndefined(dpc.minLabel);
            dpc.maxLabelAvailable = !_.isUndefined(dpc.maxLabel);
            dpc.readOnly = _.isUndefined(dpc.readOnly) ? false : dpc.readOnly === true || dpc.readOnly == "true";
            dpc.isDisabled = _.isUndefined(dpc.isDisabled) ? false : dpc.isDisabled === true || dpc.isDisabled == "true";
            dpc.enforceMinValue = _.isUndefined(dpc.enforceMinValue) ? false : dpc.enforceMinValue === true || dpc.enforceMinValue == "true";
            dpc.enforceMaxValue = _.isUndefined(dpc.enforceMaxValue) ? false : dpc.enforceMaxValue === true || dpc.enforceMaxValue == "true";
            setupDatepickerOptions();
            if (!_.isUndefined(dpc.minValue)) {
                if (dpc.minValue instanceof Date) {
                    dpc.selectedMinDate = dpc.minValue;
                } else if (moment(dpc.minValue, moment.ISO_8601, true).isValid()) {
                    dpc.selectedMinDate = new Date(moment(dpc.minValue, moment.ISO_8601, true).toISOString());
                } else {
                    var verifyDate = new Date(dpc.minValue);
                    if (dpc.minValue !== null && verifyDate instanceof Date && !isNaN(verifyDate.getTime())) {
                        dpc.selectedMinDate = verifyDate;
                    } else {
                        dpc.selectedMinDate = null;
                    }
                }
                dpc.initMin = !_.isUndefined(dpc.datePickerOptions.maxDate) && moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).isBefore(new Date().toISOString()) ? moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).toDate() : new Date();
            }
            if (!_.isUndefined(dpc.maxValue)) {
                if (dpc.maxValue instanceof Date) {
                    dpc.selectedMaxDate = new Date(dpc.maxValue.toISOString());
                } else if (moment(dpc.maxValue, moment.ISO_8601, true).isValid()) {
                    dpc.selectedMaxDate = new Date(moment(dpc.maxValue, moment.ISO_8601, true).toISOString());
                } else {
                    var verifyDate = new Date(dpc.maxValue);
                    if (dpc.maxValue !== null && verifyDate instanceof Date && !isNaN(verifyDate.getTime())) {
                        dpc.selectedMaxDate = verifyDate;
                    } else {
                        dpc.selectedMaxDate = null;
                    }
                }
                dpc.initMax = !_.isUndefined(dpc.datePickerOptions.maxDate) && moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).isBefore(new Date().toISOString()) ? moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).toDate() : new Date();
            }
            dpc.previousValue.min = angular.copy(dpc.selectedMinDate);
            dpc.previousValue.max = angular.copy(dpc.selectedMaxDate);
        }
        function openDatePicker(popup) {
            dpc.isOpenMin = popup === "min";
            dpc.isOpenMax = popup === "max";
            if (dpc.isOpenMin) {
                $timeout(function() {
                    $timeout(function() {
                        angular.element("div.datePickerMin").find("ul.uib-datepicker-popup button.uib-datepicker-current").focus();
                        angular.element("div.datePickerMin").find("ul.uib-datepicker-popup button.uib-datepicker-current").blur();
                        angular.element("div.datePickerMin").find("table").focus();
                    });
                });
            } else if (dpc.isOpenMax) {
                $timeout(function() {
                    $timeout(function() {
                        angular.element("div.datePickerMax").find("ul.uib-datepicker-popup button.uib-datepicker-current").focus();
                        angular.element("div.datePickerMax").find("ul.uib-datepicker-popup button.uib-datepicker-current").blur();
                        angular.element("div.datePickerMax").find("table").focus();
                    });
                });
            }
        }
        function setupDatepickerOptions() {
            if (_.isUndefined(dpc.datePickerOptions)) {
                dpc.datePickerOptions = {};
                return;
            } else {
                _.each(dpc.datePickerOptions, function(option, optionName) {
                    if (_.isUndefined(option) || _.isEmpty(option)) {
                        delete dpc.datePickerOptions[optionName];
                    }
                });
            }
        }
        function executeOnChange(input) {
            if (dpc.enforceMinValue && input === "min" && (_.isUndefined(dpc.selectedMinDate) || !moment(dpc.selectedMinDate, moment.ISO_8601, true).isValid())) {
                dpc.selectedMinDate = moment("1970", "YYYY").toDate();
            }
            if (dpc.enforceMaxValue && input === "max" && (_.isUndefined(dpc.selectedMaxDate) || !moment(dpc.selectedMaxDate, moment.ISO_8601, true).isValid())) {
                dpc.selectedMaxDate = moment().toDate();
            }
            if (input === "min" && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format) && !_.isUndefined(dpc.selectedMaxDate) && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format)) {
                if (moment(dpc.selectedMinDate).isAfter(moment(dpc.selectedMaxDate).toISOString())) {
                    dpc.selectedMinDate = uibDateParser.parse(new Date(dpc.previousValue.min));
                }
            }
            if (input === "max" && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format) && !_.isUndefined(dpc.selectedMinDate) && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format)) {
                if (moment(dpc.selectedMaxDate).isBefore(moment(dpc.selectedMinDate).toISOString())) {
                    dpc.selectedMaxDate = uibDateParser.parse(new Date(dpc.previousValue.max));
                }
            }
            if (input === "min" && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format)) {
                dpc.previousValue.min = angular.copy(dpc.selectedMinDate);
            }
            if (input === "max" && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format)) {
                dpc.previousValue.max = angular.copy(dpc.selectedMaxDate);
            }
            if (input === "min" && !_.isUndefined(dpc.selectedMinDate)) {
                $timeout(function() {
                    $scope.$apply(function() {
                        dpc.minValue = dpc.selectedMinDate;
                    });
                });
            }
            if (input === "max" && !_.isUndefined(dpc.selectedMaxDate)) {
                $timeout(function() {
                    $scope.$apply(function() {
                        dpc.maxValue = dpc.selectedMaxDate;
                    });
                });
            }
        }
        function convertFormatToUibDateFormat(format) {
            if (format == null || format.trim().length == 0) {
                format = DEFAULT_FORMAT;
            }
            var separator = dpc.parserSeparator != null && dpc.parserSeparator.trim().length > 0 ? [ dpc.parserSeparator ] : [ "-", "/" ];
            var uibFormat = "";
            var separatorDetected = false;
            for (var i in separator) {
                if (format.indexOf(separator[i]) !== -1) {
                    var sections = format.split(separator[i]);
                    for (var j in sections) {
                        if (sections[j].toLowerCase().indexOf("y") !== -1) {
                            uibFormat += sections[j].toLowerCase() + (j < sections.length - 1 ? separator[i] : "");
                        } else if (sections[j].toLowerCase().indexOf("m") !== -1) {
                            uibFormat += sections[j].toUpperCase() + (j < sections.length - 1 ? separator[i] : "");
                        } else if (sections[j].toLowerCase().indexOf("d") !== -1) {
                            uibFormat += sections[j].toLowerCase() + (j < sections.length - 1 ? separator[i] : "");
                        }
                    }
                    separatorDetected = true;
                    break;
                }
            }
            if (!separatorDetected) {
                if (format.toLowerCase().indexOf("y") !== -1) {
                    uibFormat += format.toLowerCase();
                } else if (format.toLowerCase().indexOf("m") !== -1) {
                    uibFormat += format.toUpperCase();
                } else if (format.toLowerCase().indexOf("d") !== -1) {
                    uibFormat += format.toLowerCase();
                }
            }
            return uibFormat;
        }
    };
    DatePickerController.$inject = [ "$scope", "$log", "i18nService", "uibDateParser", "$timeout", "ffCommonUseFactory" ];
    var dateValidator = function($log, ffCommonUseFactory) {
        var directive = {
            restrict: "A",
            require: [ "^ngModel" ],
            link: linkFunc
        };
        return directive;
        function linkFunc(scope, el, attr, ctrl) {
            var model = ctrl[0];
            var fieldId = attr.fieldId;
            var caseName = attr.caseName;
            model.$validators[attr.ffDateValidator] = function() {
                var isValid = ffCommonUseFactory.parseDate(model.$viewValue);
                scope.$emit("requiredValueChanged", {
                    isValid: isValid,
                    fieldId: fieldId,
                    caseName: caseName,
                    validator: attr.ffDateValidator
                });
                return isValid;
            };
        }
    };
    angular.module("formFactory").directive("ffDateValidator", [ "$log", "ffCommonUseFactory", dateValidator ]);
})();

(function() {
    var editInPlace = function() {
        var directive = {
            restrict: "E",
            scope: {
                value: "="
            },
            template: '<span ng-click="edit()" ng-bind="value"></span><input class="form-control" ng-model="value" />',
            link: linkFunction
        };
        return directive;
        function linkFunction(scope, el, attrs) {
            var inputElement = el.find("input");
            el.addClass("edit-in-place");
            el.addClass("inputHidden");
            scope.edit = function() {
                el.addClass("spanHidden");
                el.removeClass("inputHidden");
                inputElement[0].focus();
            };
            inputElement.on("blur", function(e) {
                e.stopPropagation();
                el.removeClass("spanHidden");
                el.addClass("inputHidden");
            });
        }
    };
    angular.module("formFactory").directive("ffEditInPlace", [ editInPlace ]);
})();

(function() {
    "use strict";
    var logicsFieldValidation = function($log, contextualData, $timeout) {
        var directive = {
            restrict: "EA",
            require: [ "^ngModel" ],
            link: linkFunc
        };
        return directive;
        function linkFunc(scope, el, attr, ctrl) {
            var type = attr["ffFieldValueValidation"];
            var fieldId = attr["fieldId"];
            var caseName = attr["caseName"];
            el.bind("keyup", function(e) {
                scope.$apply(function() {
                    validate(e.currentTarget.value);
                });
            });
            el.bind("change", function(e) {
                scope.$apply(function() {
                    validate(e.currentTarget.value);
                });
            });
            $timeout(function() {
                validate(ctrl[0].$viewValue);
            }, 500);
            function validate(value) {
                switch (type) {
                  case "any":
                    if (value === undefined || value === "" || value === null) {
                        scope.$emit("requiredValueChanged", {
                            isValid: false,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    } else {
                        scope.$emit("requiredValueChanged", {
                            isValid: true,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    }
                    break;

                  case "number":
                    if (value === undefined || value === "" || value === null || isNaN(value)) {
                        scope.$emit("requiredValueChanged", {
                            isValid: false,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    } else {
                        scope.$emit("requiredValueChanged", {
                            isValid: true,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    }
                    break;

                  case "positiveNumber":
                    if (value === undefined || value === "" || value === null || isNaN(value) || value < 0) {
                        scope.$emit("requiredValueChanged", {
                            isValid: false,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    } else {
                        scope.$emit("requiredValueChanged", {
                            isValid: true,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    }
                    break;

                  case "multiple":
                    scope.$emit("requiredValueChanged", {
                        isValid: scope.vsc.selectedValues.length > 0,
                        "case": type,
                        fieldId: fieldId,
                        caseName: caseName
                    });
                    break;
                }
            }
        }
    };
    angular.module("formFactory").directive("ffFieldValueValidation", [ "$log", "contextualData", "$timeout", logicsFieldValidation ]);
})();

(function() {
    "use strict";
    var ResolveTemplatesService = function($log) {
        this.resolveTemplatePath = function(path, viewType) {
            viewType = viewType !== undefined ? "." + viewType : "";
            return path.replace(".html.ajax", viewType + ".html.ajax");
        };
    };
    ResolveTemplatesService.$inject = [ "$log" ];
    angular.module("formFactory.templateResolver", []).service("ffTemplateResolver", ResolveTemplatesService);
})();

(function() {
    "use strict";
    var sanitize = function($sce) {
        return function(htmlCode) {
            return $sce.trustAsHtml(htmlCode);
        };
    };
    var removeSpaces = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.replace(/[\s]/g, "");
        };
    };
    var lowercase = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.toLowerCase();
        };
    };
    var normalize = function() {
        return function(value) {
            var newValue = "";
            for (var i = 0; i < value.length; i++) {
                if (value.charAt(i) === "-") {
                    if (i++ < value.length) {
                        newValue += value.charAt(i).toUpperCase();
                    }
                    continue;
                }
                newValue += value.charAt(i);
            }
            return newValue;
        };
    };
    var removeDashes = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.replace(/[\-]/g, "");
        };
    };
    var callUnderscoreString = function() {
        return function(str, fn, params) {
            str = str || "";
            params = params || [];
            params.unshift(str);
            return fn ? s[fn].apply(this, params) : str;
        };
    };
    var momentCalendar = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return moment(string, "LLL").calendar();
        };
    };
    var resourceBundle = function(i18n) {
        return function(string, resourcePartial) {
            return i18n.message(resourcePartial + "." + string);
        };
    };
    var removeChar = function() {
        return function(string, charIndex) {
            return string.substr(charIndex);
        };
    };
    var sanitizeFormName = function() {
        return function(formName) {
            if (!angular.isString(formName)) {
                return formName;
            }
            return formName.replace(/[`\s\'\"~+=!*&%#@,:;\.\^\$\?<\>\|\/\\\[\]\(\)\}\{\-]/g, "");
        };
    };
    var roundValue = function() {
        return function(value) {
            var regexp = new RegExp(/^-{0,1}\d*\.?\d*$/);
            if (!regexp.test(value)) {
                return value;
            }
            return Math.round(value);
        };
    };
    var dasherize = function() {
        return function(str) {
            return str != null ? str.toLowerCase().replace(/\s/g, "-") : "";
        };
    };
    angular.module("formFactory").filter("sanitize", [ "$sce", sanitize ]).filter("removeSpaces", [ removeSpaces ]).filter("lowercase", [ lowercase ]).filter("normalize", [ normalize ]).filter("removeDashes", [ removeDashes ]).filter("s", [ callUnderscoreString ]).filter("momentCalendar", [ momentCalendar ]).filter("i18n", [ "i18nService", resourceBundle ]).filter("removeIndex", [ removeChar ]).filter("sanitizeFormName", [ sanitizeFormName ]).filter("roundValue", [ roundValue ]).filter("dasherize", [ dasherize ]);
})();

(function() {
    "use strict";
    var i18nMessageKeyDirectiveFunction = function(i18nService) {
        return {
            restrict: "A",
            link: function($scope, $element, $attrs) {
                var i18n;
                if (!$attrs.messageParams) {
                    i18n = i18nService.message($attrs.messageKey);
                } else {
                    i18n = i18nService.format($attrs.messageKey, $attrs.messageParams);
                }
                if ($attrs.messageAttr) {
                    $element.attr($attrs.messageAttr, i18n);
                } else {
                    $element.text(i18n);
                }
            }
        };
    };
    var i18nTranslateFilterFunction = function(i18nService) {
        return function(input) {
            return i18nService.message(input);
        };
    };
    angular.module("i18n", []).service("i18nService", function() {
        this.message = function(key, fallback) {
            if (ffi18n && ffi18n[key]) {
                return ffi18n[key];
            } else if (angular.isDefined(fallback)) {
                return fallback;
            } else {
                return "???" + key + "???";
            }
        };
        this.format = function(key, params) {
            var replacer = function(params) {
                return function(s, index) {
                    return params[index] ? params[index] === "__void__" ? "" : params[index] : "";
                };
            };
            if (params) {
                if (ffi18n && ffi18n[key]) {
                    return ffi18n[key].replace(/\{(\w+)\}/g, replacer(params.split("|")));
                } else {
                    return "???" + key + "???";
                }
            } else {
                return this.message(key);
            }
        };
    }).directive("messageKey", [ "i18nService", i18nMessageKeyDirectiveFunction ]).filter("translate", [ "i18nService", i18nTranslateFilterFunction ]);
})();

(function() {
    "use strict";
    var UnderscoreFactory = function() {
        return window._;
    };
    angular.module("formFactory").factory("_", UnderscoreFactory);
})();